// Buffer class header
#pragma once

#include "Resource.h"
#include "VideoMemoryAllocator.h"

namespace nf::client::render {
    enum class BufferType {
        Vertex,
        Index,
        Uniform
    };

    class Buffer : Resource {
    public:
        Buffer(BufferType type, const VkDevice& device, VideoMemoryAllocator& allocator, const VkCommandPool& commandPool, const VkQueue& queue, void* bufferData, size_t bufferSize);

        const VkBuffer& getHandle() const;
        void* getPointer() const;

        ~Buffer();
    private:
        void createBuffer(VkDeviceSize size, VkBufferUsageFlags usage, VkBuffer& buffer);
        void copyBuffer(VkBuffer sourceBuffer, VkBuffer destinationBuffer, VkDeviceSize size, VkCommandPool commandPool, VkQueue queue);

        void destroyBuffer(VkBuffer buffer, VkDeviceMemory bufferMemory, VkDeviceSize offset);

        VideoMemoryAllocator& m_allocator;

        VkBuffer m_handle;
        VkDeviceMemory m_memory;
        VkDeviceSize m_offset;
        void* m_pointer;
    };
}
