// VideoMemoryAllocator class header
#pragma once

namespace nf::client::render {
    struct MemoryBlock {
        VkDeviceMemory memory;
        uint32_t memoryTypeIndex;
        size_t size;

        std::map<VkDeviceSize, VkDeviceSize> allocations;
        //       Offset        Size
    };

    class VideoMemoryAllocator {
    public:
        VideoMemoryAllocator(const VkDevice& device, const VkPhysicalDevice& physicalDevice);

        void allocateForBuffer(VkBuffer buffer, VkMemoryPropertyFlags memoryPropertyFlags, VkDeviceMemory& outDeviceMemory, VkDeviceSize& outOffset);
        void deallocate(VkDeviceMemory memoryBlock, VkDeviceSize offset);
    private:
        uint32_t findMemoryType(uint32_t typeFilter, VkMemoryPropertyFlags desiredPropertyFlags);

        void allocateFromBlock(VkMemoryAllocateInfo& allocateInfo, uint32_t alignment, VkDeviceMemory& outDeviceMemory, VkDeviceSize& outOffset);
        size_t calculateBlockSize(uint32_t memoryTypeIndex) const;
        void allocateBlock(VkMemoryAllocateInfo& allocateInfo);

        const VkDevice& m_device;
        const VkPhysicalDevice& m_physicalDevice;

        std::vector<MemoryBlock> m_blocks;
    };
}
